/*
  PromptNuFlux flux_interpolation.cc
  Author: Luca Rottoli, 2015
*/

#include "flux_interpolation.hh"

using namespace std;

/*
	Constructor
 */
FluxInterpolation::FluxInterpolation()
{
	fNPoints = 46;
	fNModels = 5;

	fSplineC = new gsl_spline * [fNModels];
	fSplineU = new gsl_spline * [fNModels];
	fSplineL = new gsl_spline * [fNModels];
	for (int i = 0; i < fNModels; ++i)
	{
		fSplineC[i] = gsl_spline_alloc (gsl_interp_linear, fNPoints);
		fSplineU[i] = gsl_spline_alloc (gsl_interp_linear, fNPoints);
		fSplineL[i] = gsl_spline_alloc (gsl_interp_linear, fNPoints);
	}

	cout << endl << "Initialization..." << endl << endl;

	string mod[] = {"BPL","H3P", "H3A", "H14A", "H14B"};

	double * energies = new double [fNPoints];

	double * central  = new double [fNPoints];
	double * upper    = new double [fNPoints];
	double * lower    = new double [fNPoints];

	string inputdir = "inputs/";
	for (int i = 0; i < fNModels; ++i)
	{
		string input = inputdir + mod[i] + ".dat"; 
		ifstream infile(input.c_str());
		if (!infile.is_open())
		{
			cout << " \033[1;31m Something wrong happened! The file " << input << " cannot be loaded! Abort. \033[0m"  << endl << endl;
			abort();            
		}
		string dummyline;
		int dummylines = 0;
		for (int k = 0; k < dummylines; ++k)
			getline(infile,dummyline);
		for (int l = 0; l < fNPoints; ++l)
		{
			infile >> energies[l]; 
			infile >> central[l];
			infile >> upper[l];
			infile >> lower[l];
			energies[l] = log10(energies[l]);
		}
		infile.close();
		gsl_spline_init (fSplineC[i], energies, central, fNPoints);
		gsl_spline_init (fSplineU[i], energies, upper,   fNPoints);
		gsl_spline_init (fSplineL[i], energies, lower,   fNPoints);
	}

	cout << endl << "Initialization completed" << endl << endl;

	delete [] upper;
	delete [] central;
	delete [] lower;
	delete [] energies;
}

/*
	Destructor
 */
FluxInterpolation::~FluxInterpolation()
{
	delete [] fSplineC;
	delete [] fSplineU;
	delete [] fSplineL;
}

/*
	Copy Constructor
*/ 
FluxInterpolation::FluxInterpolation(const FluxInterpolation& other):
fNPoints(other.fNPoints), fNModels(other.fNModels)
{
	fSplineC = new gsl_spline * [fNModels];
	fSplineU = new gsl_spline * [fNModels];
	fSplineL = new gsl_spline * [fNModels];
	for (int i = 0; i < fNModels; ++i)
	{
		fSplineC[i] = other.fSplineC[i];
		fSplineU[i] = other.fSplineU[i];
		fSplineL[i] = other.fSplineL[i];
	}
}

/*
	CentralFlux(E,flux) returns the central prediction for a given energy E and a cosmic ray flux model flux.
 */
double FluxInterpolation::CentralFlux(double E, fluxmodel flux)
{
	double result;
	if(E < 1e3 || E > pow(10,7.5) )
	{
		cout << "\033[1;31m Wrong value of E. Accepted values: 1e3 < E < 1e7.5. Abort. \033[0m" << endl << endl;
		abort();
	}
	gsl_interp_accel *acc = gsl_interp_accel_alloc ();
	result = gsl_spline_eval(fSplineC[(int) flux], log10(E), acc);
	return result;
}

/*
	UpperFlux(E,flux) returns the lower prediction for a given energy E and a cosmic ray flux model flux.
 */
double FluxInterpolation::UpperFlux(double E, fluxmodel flux)
{
	double result;
	if(E < 1e3 || E > pow(10,7.5) )
	{
		cout << " \033[1;31m Wrong value of E. Accepted values: 1e3 < E < 1e7.5. Abort. \033[0m" << endl << endl;
		abort();
	}
	gsl_interp_accel *acc = gsl_interp_accel_alloc ();
	result = gsl_spline_eval(fSplineU[(int) flux], log10(E), acc);
	return result;
}

/*
	LowerFlux(E,flux) returns the upper prediction for a given energy E and a cosmic ray flux model flux.
 */
double FluxInterpolation::LowerFlux(double E, fluxmodel flux)
{
	double result;
	if(E < 1e3 || E > pow(10,7.5) )
	{
		cout << "\033[1;31m Wrong value of E. Accepted values: 1e3 < E < 1e7.5. Abort. \033[0m" << endl << endl;
		abort();
	}
	gsl_interp_accel *acc = gsl_interp_accel_alloc ();
	result = gsl_spline_eval(fSplineL[(int) flux], log10(E), acc);
	return result;
}
